<?php

namespace App\DataTables\backend;

use App\Models\Booking;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;

class BookingsDataTable extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */
    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        return (new EloquentDataTable($query))
            ->addColumn('#', function () {
                static $rowNumber = 0; // Static variable to maintain state
                return ++$rowNumber;   // Increment and return the row number
            })
            ->addColumn('action', function ($booking) {
                return '<a href="'.route('booking.edit', $booking->id).'" target="_blank" class="btn btn-sm btn-primary edit-booking-btn"><i class="bi bi-pencil-square"></i></a>';
            })
            ->addColumn('block_name', function ($row) {
                if (is_array($row)) {
                    return $row['block']['name'] ?? 'N/A';
                } else {
                    return optional($row->block)->name ?? 'N/A';
                }
            })
            ->rawColumns(['block_name', 'action'])
            ->setRowId('id');
    }

    /**
     * Get the query source of dataTable.
     */
    public function query(Booking $model): QueryBuilder
    {
        $query = $model->newQuery()->with(['product', 'company', 'airport', 'customer', 'block']);

        // Apply filters
        if (request()->has('product_id') && request('product_id') != '') {
            $query->where('product_id', request('product_id'));
        }

        if (request()->has('date_filter_id') && request('date_filter_id') != '') {
            // Add logic for date filter
        }

        if (request()->has('agent_id') && request('agent_id') != '') {
            $query->where('company_id', request('agent_id'));
        }

        if (request()->has('airport_id') && request('airport_id') != '') {
            $query->where('airport_id', request('airport_id'));
        }

        // Apply date filter based on the selected type and date range
        if (request()->has('date_filter_type') && request()->has('date_from') && request()->has('date_to')) {
            $dateFilterType = request()->input('date_filter_type');
            $dateFrom = request()->input('date_from');
            $dateTo = request()->input('date_to');

            switch ($dateFilterType) {
                case 1: // Booking Date
                    $query->whereBetween('booking_date', [$dateFrom, $dateTo]);
                    break;
                case 2: // Pick Date
                    $query->whereBetween('date_to', [$dateFrom, $dateTo]);
                    break;
                case 3: // Drop Date
                    $query->whereBetween('date_from', [$dateFrom, $dateTo]);
                    break;
            }
        }

        if (request()->has('terminal_id') && request('terminal_id') != '') {
            $query->where('terminal_id', request('terminal_id'));
        }

        if (request()->has('payment_status') && request('payment_status') != '') {
            $query->where('payment_status', request('payment_status'));
        }

        if (request()->has('search') && request('search') != '') {
            $search = request('search');
            $query->where(function($q) use ($search) {
                $q->where('name', 'like', "%$search%")
                    ->orWhere('ref_no', 'like', "%$search%")
                    ->orWhere('email', 'like', "%$search%")
                    ->orWhere('mobile', 'like', "%$search%")
                    ->orWhere('reg_no', 'like', "%$search%");
            });
        }

        return $query;
    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
                    ->setTableId('bookings-table')
                    ->columns($this->getColumns())
                    ->minifiedAjax()
                    ->dom("<'row'<'col-sm-12 col-md-4'l><'col-sm-12 col-md-4'B><'col-sm-12 col-md-4'f>>" .  // Custom layout for length menu, buttons, and search bar
                        "<'row'<'col-sm-12'tr>>" .
                        "<'row'<'col-sm-12 col-md-5'i><'col-sm-12 col-md-7'p>>") // Pagination on the right
                    ->orderBy(1)
                    ->selectStyleSingle()
                    ->addTableClass('table table-bordered table-hover dataTable dtr-inline')
                    ->buttons([
                        Button::make('excel'),
                        Button::make('csv'),
                        Button::make('pdf'),
                        Button::make('colvis'),
                    ]);
    }

    /**
     * Get the dataTable columns definition.
     */
    public function getColumns(): array
    {
        return [
            Column::make('#'),
            Column::make('company.name'),
            Column::make('airport.name'),
            Column::make('user_id'),
            Column::make('customer.first_name'),
            Column::make('product.name'),
            Column::make('block_name'),
            Column::make('booking_date'),
            Column::make('date_from'),
            Column::make('date_to'),
            Column::make('dis_code'),
            Column::make('price'),
            Column::make('net_price'),
            Column::make('status'),
            Column::make('pay_at_exit'),
            Column::make('payment_status'),
            Column::make('payment_ref'),
            Column::make('refunded'),
            Column::make('cancel_date'),
            Column::make('cancel_reason'),
            Column::make('delay_charge'),
            Column::make('remarks'),
            Column::make('tax_amount'),
            Column::make('discount_amount'),
            Column::make('total_amount'),
            Column::make('promotion_code'),
            Column::computed('action')
                ->exportable(false)
                ->printable(false)
                ->width(60)
                ->addClass('text-center')
        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'Bookings_' . date('YmdHis');
    }
}
