<?php

namespace App\DataTables\backend;

use App\Models\Company;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;

class CompaniesDataTable extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */
    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        return (new EloquentDataTable($query))
            ->addColumn('#', function () {
                static $rowNumber = 0; // Static variable to maintain state
                return ++$rowNumber;   // Increment and return the row number
            })
            ->addColumn('action', function ($company) {
                return '<a data-id="'.$company->id.'" class="btn btn-sm btn-primary edit-company-btn"><i class="bi bi-pencil-square"></i></a>
                        <a data-id="'.$company->id.'" class="btn btn-sm btn-danger delete-company-btn"><i class="bi bi-trash3-fill"></i></a>';
            })
            ->addColumn('logo', function ($company) {
                return '<img src="' . $company->logo . '" width="50" height="50" class="img-thumbnail">';
            })
            ->rawColumns(['logo', 'action'])
            ->setRowId('id');
    }

    /**
     * Get the query source of dataTable.
     */
    public function query(Company $model): QueryBuilder
    {
        return $model->newQuery();
    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
                    ->setTableId('companies-table')
                    ->columns($this->getColumns())
                    ->minifiedAjax()
                    //->dom('Bfrtip')
                    ->orderBy(1)
                    ->selectStyleSingle()
                    ->addTableClass('table table-bordered table-hover dataTable dtr-inline')
                    ->buttons([
                        Button::make('excel'),
                        Button::make('csv'),
                        Button::make('pdf'),
                        Button::make('print')
                    ]);
    }

    /**
     * Get the dataTable columns definition.
     */
    public function getColumns(): array
    {
        return [
            Column::make('#'),
            Column::make('name'),
            Column::make('register_number'),
            Column::make('vat_number'),
            Column::make('street'),
            Column::make('city'),
            Column::make('hotline_1'),
            Column::make('company_email'),
            Column::make('website'),
            Column::computed('logo')
                ->title('Logo')
                ->exportable(false)
                ->printable(false)
                ->width(100)
                ->addClass('text-center'),
            Column::computed('action')
                ->exportable(false)
                ->printable(false)
                ->width(60)
                ->addClass('text-center')
        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'Companies_' . date('YmdHis');
    }
}
