<?php

namespace App\Http\Controllers;

use App\Models\Booking;
use App\Models\Customer;
use App\Notifications\BookingConfirmationEmail;
use App\Notifications\InvoicePaid;
use App\Services\AirportService;
use App\Services\CustomerService;
use App\Services\InvoiceService;
use App\Services\ProductService;
use App\Services\VehicleService;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Requests\StoreBookingRequest;
use App\Services\BookingService;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Notification;
use App\Http\Controllers\Controller;

class BookingController extends Controller
{
    protected $bookingService;
    private ProductService $productService;
    private AirportService $airportService;
    private VehicleService $vehicleService;
    private CustomerService $customerService;
    private InvoiceService $invoiceService;

    public function __construct(BookingService $bookingService, ProductService $productService,
                                AirportService $airportService,
                                VehicleService $vehicleService,
                                CustomerService $customerService,
                                InvoiceService $invoiceService
    )
    {
        $this->bookingService = $bookingService;
        $this->productService = $productService;
        $this->airportService = $airportService;
        $this->vehicleService = $vehicleService;
        $this->customerService = $customerService;
        $this->invoiceService = $invoiceService;
    }

    /**
     * Display a listing of the resource.
     */
    public function index(): JsonResponse
    {
        $bookings = $this->bookingService->listBookings(request()->all());
        return response()->json($bookings, 200);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(Request $request)
    {
        $filters = $request->query();

        if (empty($filters['package']) OR empty($filters['airport'])) {
            return redirect()->route('home');
        }

        // Perform the search
        $package = $this->productService->getProductAssignAirportById($filters['package'], $filters['airport']);

        // todo: check weather the airport terminals separated to arrival and departure
        $terminals = $this->airportService->terminals($filters['airport']);

        $vehicleMakes = $this->vehicleService->vehicleMakes();

        $filters['date_from'] = Carbon::parse($filters['dropoff_date'].$filters['dropoff_time'])->format('Y-m-d H:i:s');
        $filters['date_to'] = Carbon::parse($filters['pickup_date'].$filters['pickup_time'])->format('Y-m-d H:i:s');

        return view('booking', compact('package','filters', 'terminals', 'vehicleMakes'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(StoreBookingRequest $request)
    {
        $validated = $request->all();
        $request->flashOnly(array_keys($validated));

        $formattedVehicleArray = $validated['vehicle'];
        $booking = null;
        DB::transaction(function () use ($validated, $request, $formattedVehicleArray, &$booking) {

            // New Customer
            $customer = $this->customerService->fetchOrCreate($validated['customer']['title'], $validated['customer']);
            $validated['booking']['customer_id'] = $customer->id;

            // New booking
            $validated['booking']['booking_date'] = Carbon::now()->format('Y-m-d H:i:s');
            $validated['booking']['net_price'] = $validated['booking']['price'];
            $validated['booking']['total_amount'] = $validated['booking']['price'];
            $validated['booking']['status'] = 'pending';
            $validated['booking']['payment_status'] = 'pending';
            $booking = $this->bookingService->createBooking($validated['booking']);

            // New vehicles
            array_walk($formattedVehicleArray, function(&$item1) use ($booking, $customer) {
                $item1['booking_id'] = $booking['id'] ?? 0;
                $item1['customer_id'] =  $booking['id'] ?? 0;
            });
            $this->vehicleService->fetchOrCreate($formattedVehicleArray);

            //generate invoice
            $validated['booking']['booking_id'] =  $booking['id'] ;
            $invoice = $this->invoiceService->storeInvoice($this->invoiceService->formatStoreData($validated['booking']));
        });

        return redirect('payment/'.$booking['id'])->with('success', 'Booking created successfully.');
    }

    public function payment($booking_id)
    {
        $booking = $this->bookingService->formatBookingResponse($booking_id);
        return view('payment', compact('booking'));
    }

    /**
     * Display the specified resource.
     */
    public function show(int $id): JsonResponse
    {
        $booking = $this->bookingService->getBookingById($id);

        if (!$booking) {
            return response()->json(['message' => 'Booking not found'], 404);
        }

        return response()->json($booking, 200);
    }


    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Booking $booking)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(StoreBookingRequest $request, int $id): JsonResponse
    {
        $booking = $this->bookingService->updateBooking($id, $request->validated());

        if (!$booking) {
            return response()->json(['message' => 'Booking not found'], 404);
        }

        return response()->json($booking, 200);
    }

    public function cancel(int $id): JsonResponse
    {
        $booking = $this->bookingService->cancelBooking($id);

        if (!$booking) {
            return response()->json(['message' => 'Booking not found'], 404);
        }

        return response()->json(['message' => 'Booking canceled successfully'], 200);
    }


    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Booking $booking)
    {
        //
    }
}
