<?php

namespace App\Http\Controllers;

use App\Models\Customer;
use App\Notifications\BookingConfirmationEmail;
use App\Services\BookingService;
use App\Services\PaymentService;
use App\Services\StripeService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Notification;
use Stripe\Stripe;
use Stripe\Webhook;
use Stripe\Exception\SignatureVerificationException;
use Stripe\Exception\UnexpectedValueException;

class StripeWebhookController extends Controller
{
    private PaymentService $paymentService;
    private StripeService $stripeService;
    private BookingService $bookingService;

    public function __construct(PaymentService $paymentService, StripeService $stripeService, BookingService $bookingService)
    {
        $this->paymentService = $paymentService;
        $this->stripeService = $stripeService;
        $this->bookingService = $bookingService;
    }

    public function handle(Request $request)
    {
        // Retrieve Stripe signature from the header
        $signature = $request->header('Stripe-Signature');
        $endpoint_secret = config('services.stripe.webhook_secret');

        try {
            // Verify the webhook signature

            //$event = Webhook::constructEvent($request->getContent(), $signature, $endpoint_secret);
            $event = $this->stripeService->webhookData($request->getContent(), $signature, $endpoint_secret);
            Log::info('Payment succeeded', ['payment_intent' => $event->data->object]);



        } catch (UnexpectedValueException $e) {
            Log::error('Invalid Stripe payload', ['error' => $e->getMessage()]);
            return response()->json(['error' => 'Invalid payload'], 400);
        } catch (SignatureVerificationException $e) {
            Log::error('Stripe signature verification failed', ['error' => $e->getMessage()]);
            return response()->json(['error' => 'Signature verification failed'], 400);
        }

        // Handle different event types
        switch ($event->type) {
            case 'payment_intent.succeeded':
                $paymentIntent['booking_id'] = 1;
                $paymentIntent['payment_id'] = 1;
                $paymentIntent['payment_log'] = $event->data->object;
                $this->paymentService->saveCheckoutSuccessLog($paymentIntent);

                Log::info('Payment succeeded', ['payment_intent' => $paymentIntent]);
                // Perform actions (e.g., update database, send email)

//                $booking = $this->bookingService->formatBookingResponse($booking_id);
//                $booking['system_logo'] = config('app.system.logo');
//                Notification::route('mail', [$booking['customer']['email'] => $booking['customer']['first_name']])
//                    ->notify(new BookingConfirmationEmail($booking));

                break;

            case 'payment_intent.failed':
                $paymentIntent = $event->data->object;
                Log::warning('Payment failed', ['payment_intent' => $paymentIntent]);
                // Handle failed payment
                break;

            default:
                Log::info('Unhandled Stripe event', ['event' => $event->type]);
        }

        return response()->json(['message' => 'Webhook received']);
    }
}
