<?php

namespace App\Http\Controllers\backend;

use App\DataTables\backend\BlocksDataTable;
use App\Http\Controllers\Controller;
use App\Services\SiteService;
use Illuminate\Http\Request;
use App\Models\Block;
use Illuminate\Support\Facades\Log;

class BlockController extends Controller
{
    public function __construct(private SiteService $siteService, private Block $model) { }
    /**
     * Display a listing of the resource.
     */
    public function index(BlocksDataTable $dataTable)
    {
        $sites = $this->siteService->listSites();

        $data = ['title' => 'Configure Block Information','sites' => $sites];
        return $dataTable->render('backend.block', $data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            // Validate the request
            $request->validate(Block::$rules);

            // Save to database
            $block = $this->model->create([
                'name' => $request->name,
                'latitude' => $request->latitude,
                'longitude' => $request->longitude,
                'capacity' => $request->capacity,
                'site_id' => $request->site_id
            ]);

            // Return success response
            return response()->json([
                'message' => 'Block saved successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error saving block: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while saving the block. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        try {
            $block = $this->model->findOrFail($id);

            return response()->json($block);

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error fetching block details: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while fetching block details. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {
            $request->validate(Block::$rules);

            $block = $this->model->findOrFail($id);

            $block->name = $request->name;
            $block->latitude = $request->latitude;
            $block->longitude = $request->longitude;
            $block->capacity = $request->capacity;
            $block->site_id = $request->site_id;
            $block->save();

            return response()->json([
                'message' => 'Block details updated successfully!',
            ]);

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error updating block: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while updating the block. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {
            $block = $this->model->findOrFail($id);
            $block->delete();

            return response()->json([
                'message' => 'Block deleted successfully!',
            ]);
        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error deleting block: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while deleting the block. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }
}
