<?php

namespace App\Http\Controllers\backend;

use App\DataTables\backend\CompaniesDataTable;
use App\Http\Controllers\Controller;
use App\Models\Company;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class CompanyController extends Controller
{
    public function __construct(private Company $model) { }
    /**
     * Display a listing of the resource.
     */
    public function index(CompaniesDataTable $dataTable)
    {
        $data = ['title' => 'Configure Company Information'];
        return $dataTable->render('backend.company', $data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            $request->validate(Company::$rules);

            // Handle file upload
            if ($request->hasFile('logo')) {
                $logoPath = $request->file('logo')->store('company/logos', 'public');
            } else {
                $logoPath = null;
            }

            // Save to database (example)
            $company = $this->model->create([
                'name' => $request->name,
                'register_number' => $request->register_number,
                'vat_number' => $request->vat_number,
                'address' => $request->address,
                'street' => $request->street,
                'city' => $request->city,
                'country' => $request->country,
                'postal_code' => $request->postal_code,
                'hotline_1' => $request->hotline_1,
                'hotline_2' => $request->hotline_2,
                'booking_email' => $request->booking_email,
                'password' => bcrypt($request->password),
                'company_email' => $request->company_email,
                'website' => $request->website,
                'logo' => $logoPath,
            ]);

            return response()->json([
                'message' => 'Company details saved successfully!',
            ]);
        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error saving company: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while saving the company. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        try {
            // Fetch the company data by ID
            $company = $this->model->findOrFail($id);

            // Return the company data as JSON
            return response()->json([
                'company' => $company,
            ]);

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error fetching company details: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while fetching company details. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {
            $request->validate(Company::$rules);

            // Find the company by ID
            $company = $this->model->findOrFail($id);

            // Handle file upload
            if ($request->hasFile('logo')) {
                // Delete the old logo if it exists
                if ($company->logo && Storage::disk('public')->exists($company->logo)) {
                    Storage::disk('public')->delete($company->logo);
                }

                // Store the new logo
                $logoPath = $request->file('logo')->store('company/logos', 'public');
                $company->logo = $logoPath;
            }

            // Update the company details
            $company->name = $request->name;
            $company->register_number = $request->register_number;
            $company->vat_number = $request->vat_number;
            $company->address = $request->address;
            $company->street = $request->street;
            $company->city = $request->city;
            $company->country = $request->country;
            $company->postal_code = $request->postal_code;
            $company->hotline_1 = $request->hotline_1;
            $company->hotline_2 = $request->hotline_2;
            $company->booking_email = $request->booking_email;
            $company->password = $request->password ? bcrypt($request->password) : $company->password;
            $company->company_email = $request->company_email;
            $company->website = $request->website;
            $company->save();

            return response()->json([
                'message' => 'Company details updated successfully!',
            ]);

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error updating company: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while updating the company. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {
            $company = $this->model->findOrFail($id); // Find the company by ID
            $company->delete(); // Soft delete the company

            return response()->json([
                'message' => 'Company deleted successfully!',
            ]);
        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error deleting company: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while deleting the company. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }
}
