<?php

namespace App\Http\Controllers\backend;

use App\DataTables\backend\ProductsDataTable;
use App\Http\Controllers\Controller;
use App\Models\AirportProducts;
use App\Services\AirportService;
use App\Services\CompanyService;
use App\Services\ProductFeatureService;
use Illuminate\Http\Request;
use App\Models\Product;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use function Pest\Laravel\json;

class ProductController extends Controller
{
    public function __construct(private CompanyService $companyService, private AirportService $airportService, private Product $model, private ProductFeatureService $productFeatureService) { }
    /**
     * Display a listing of the resource.
     */
    public function index(ProductsDataTable $dataTable)
    {
        $companies = $this->companyService->listCompanies();
        $airports = $this->airportService->listAirports();
        $features = $this->productFeatureService->listProductFeatures();
        $selectedFeatures = [];

        $data = ['title' => 'Configure Product Information','companies' => $companies,'airports' => $airports,'features' => $features, 'selectedFeatures' => $selectedFeatures];
        return $dataTable->render('backend.product', $data);

    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            $request->validate([
                'name' => 'required|string|max:150',
                'featured_image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
                'terms_and_conditions' => 'required|string',
                'description' => 'required|string',
                'drop_off_procedure' => 'required|string',
                'pick_up_procedure' => 'required|string',
                'min_dates' => 'required|integer|min:1',
                'max_dates' => 'required|integer|min:1|gte:min_dates',
                'company_id' => 'required|exists:companies,id',
                'airport_id' => 'required|exists:airports,id',
                'price' => 'required|numeric|min:0', // Use 'numeric' instead of 'double'
                'quantity' => 'required|integer|min:0', // Changed 'quantity' to 'available_quantity'
                'features' => 'required|array', // Ensure 'features' is an array
                'is_active' => 'required|boolean',
            ]);

            // Start a database transaction
            DB::beginTransaction();

            // Handle file upload
            $featuredImagePath = null;
            if ($request->hasFile('featured_image')) {
                $featuredImagePath = $request->file('featured_image')->store('products', 'public'); // Save to public/products directory
            }

            $product = $this->model->create([
                'name' => $request->name,
                'min_dates' => $request->min_dates,
                'max_dates' => $request->max_dates,
                'company_id' => $request->company_id,
                'description' => $request->description,
                'drop_off_procedure' => $request->drop_off_procedure,
                'pick_up_procedure' => $request->pick_up_procedure,
                'terms_and_conditions' => $request->terms_and_conditions,
                'featured_image' => $featuredImagePath,
                'feature_ids' => json_encode($request->features)
            ]);

            $airportProduct = AirportProducts::create([
               'product_id' => $product->id,
               'airport_id' => $request->airport_id,
               'is_active' => $request->is_active,
               'price' => $request->price,
               'available_quantity' => $request->quantity
            ]);

            DB::commit();

            // Return success response
            return response()->json([
                'message' => 'Product saved successfully!',
                'product' => $product, // Optionally return the saved product
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            DB::rollBack();
            // Log the error for debugging
            Log::error('Error saving product: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while saving the product. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        try {
            $product = $this->model->findOrFail($id);
            $airportProduct = AirportProducts::where('product_id', $id)->first();

            if (!$airportProduct) {
                return response()->json([
                    'message' => 'Associated airport product not found.',
                ], 404); // HTTP 404 Not Found
            }

            $responseData = [
                'product' => $product,
                'airport_product' => $airportProduct,
            ];

            return response()->json([
                'message' => 'Product retrieved successfully!',
                'data' => $responseData,
            ], 200);

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error fetching product details: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while fetching product details. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {
            $request->validate([
                'name' => 'required|string|max:150',
                'featured_image' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
                'terms_and_conditions' => 'required|string',
                'description' => 'required|string',
                'drop_off_procedure' => 'required|string',
                'pick_up_procedure' => 'required|string',
                'min_dates' => 'required|integer|min:1',
                'max_dates' => 'required|integer|min:1|gte:min_dates',
                'company_id' => 'required|exists:companies,id',
                'airport_id' => 'required|exists:airports,id',
                'price' => 'required|numeric|min:0', // Use 'numeric' instead of 'double'
                'quantity' => 'required|integer|min:0', // Changed 'quantity' to 'available_quantity'
                'features' => 'required|array', // Ensure 'features' is an array
                'is_active' => 'required|boolean',
            ]);

            DB::beginTransaction();

            $product = $this->model->findOrFail($id); // Find the product by ID

            // Handle file upload
            if ($request->hasFile('featured_image')) {
                // Delete the old image if it exists
                if ($product->featured_image && Storage::disk('public')->exists($product->featured_image)) {
                    Storage::disk('public')->delete($product->featured_image);
                }

                // Store the new image
                $featuredImagePath = $request->file('featured_image')->store('products', 'public');
                $product->featured_image = $featuredImagePath;
            }

            // Update product data
            $product->name = $request->name;
            $product->min_dates = $request->min_dates;
            $product->max_dates = $request->max_dates;
            $product->company_id = $request->company_id;
            $product->description = $request->description;
            $product->drop_off_procedure = $request->drop_off_procedure;
            $product->pick_up_procedure = $request->pick_up_procedure;
            $product->terms_and_conditions = $request->terms_and_conditions;
            $product->feature_ids = json_encode($request->features);
            $product->save();

             // Update or create the associated airport product
            $airportProduct = AirportProducts::updateOrCreate(
                ['product_id' => $product->id], // Search condition
                [
                    'airport_id' => $request->airport_id,
                    'is_active' => $request->is_active,
                    'price' => $request->price,
                    'available_quantity' => $request->quantity,
                ]
            );

            // Commit the transaction
            DB::commit();

            return response()->json([
                'message' => 'Product updated successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Error updating product: ' . $e->getMessage());
            return response()->json([
                'message' => 'An error occurred while updating the product. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {
            $product = $this->model->findOrFail($id); // Find the product by ID
            $product->delete(); // Soft delete the product

            return response()->json([
                'message' => 'Product deleted successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Exception $e) {
            Log::error('Error deleting product: ' . $e->getMessage());
            return response()->json([
                'message' => 'An error occurred while deleting the product. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }
}
