<?php

namespace App\Http\Controllers\backend;

use App\DataTables\backend\SitesDataTable;
use App\Http\Controllers\Controller;
use App\Services\AirportService;
use App\Services\CompanyService;
use Illuminate\Http\Request;
use App\Models\Site;
use Illuminate\Support\Facades\Log;

class SiteController extends Controller
{
    public function __construct(private AirportService $airportService, private CompanyService $companyService, private Site $model)
    {
    }

    /**
     * Display a listing of the resource.
     */
    public function index(SitesDataTable $dataTable)
    {
        $airports = $this->airportService->listAirports();
        $companies = $this->companyService->listCompanies();

        $data = ['title' => 'Configure Site Information', 'airports' => $airports, 'companies' => $companies];
        return $dataTable->render('backend.site', $data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            // Validate the request
            $request->validate(Site::$rules);

            // Save to database
            $site = $this->model->create([
                'name' => $request->name,
                'company_id' => $request->company_id,
                'airport_id' => $request->airport_id,
            ]);

            // Return success response
            return response()->json([
                'message' => 'Site saved successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error saving site: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while saving the site. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        try {
            // Fetch the company data by ID
            $site = $this->model->findOrFail($id);

            // Return the company data as JSON
            return response()->json($site);

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error fetching site details: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while fetching site details. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {
            $request->validate(Site::$rules);

            // Find the company by ID
            $site = $this->model->findOrFail($id);

            // Update the company details
            $site->name = $request->name;
            $site->company_id = $request->company_id;
            $site->airport_id = $request->airport_id;
            $site->save();

            return response()->json([
                'message' => 'Site details updated successfully!',
            ]);

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error updating site: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while updating the site. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {
            $site = $this->model->findOrFail($id); // Find the company by ID
            $site->delete(); // Soft delete the company

            return response()->json([
                'message' => 'Site deleted successfully!',
            ]);
        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error deleting site: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while deleting the site. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }
}
