<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class ContactPerson extends Model
{
    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'contact_people';

    /**
     * The attributes that are mass assignable.
     *
     * @var array<string>
     */
    protected $fillable = [
        'email',
        'company_id',
        'phone_no_1',
        'phone_no_2',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email' => 'string',
        'phone_no_1' => 'string',
        'phone_no_2' => 'string',
    ];

    /**
     * Get the validation rules for the model.
     *
     * @return array<string, mixed>
     */
    public static function validationRules(): array
    {
        return [
            'email' => ['required', 'email', 'max:100', 'unique:contact_people'],
            'company_id' => ['required', 'exists:companies,id'],
            'phone_no_1' => ['required', 'string', 'max:20', 'regex:/^\+[1-9]\d{1,14}$/'],
            'phone_no_2' => ['nullable', 'string', 'max:20', 'regex:/^\+[1-9]\d{1,14}$/'],
        ];
    }

    /**
     * Get the company that owns the contact person.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function company(): BelongsTo
    {
        return $this->belongsTo(Company::class);
    }

    /**
     * Format phone number to international format.
     *
     * @param string $phoneNumber
     * @return string
     */
    protected function formatPhoneNumber(string $phoneNumber): string
    {
        // Remove all non-numeric characters except +
        $cleaned = preg_replace('/[^\d+]/', '', $phoneNumber);

        // Ensure it starts with +
        return str_starts_with($cleaned, '+') ? $cleaned : "+{$cleaned}";
    }

    /**
     * Set the primary phone number.
     *
     * @param string $value
     * @return void
     */
    public function setPhoneNo1Attribute($value): void
    {
        $this->attributes['phone_no_1'] = $this->formatPhoneNumber($value);
    }

    /**
     * Set the secondary phone number.
     *
     * @param string|null $value
     * @return void
     */
    public function setPhoneNo2Attribute($value): void
    {
        $this->attributes['phone_no_2'] = $value ? $this->formatPhoneNumber($value) : null;
    }

    /**
     * Scope a query to filter contact people by company.
     *
     * @param \Illuminate\Database\Eloquent\Builder $query
     * @param int $companyId
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function scopeForCompany($query, int $companyId)
    {
        return $query->where('company_id', $companyId);
    }
}
